@@ -1,3 +1,5 @@
+import json
+import logging
 import os
 import pydoc
 import shutil
@@ -5,6 +7,7 @@ import subprocess
 import sys
 from builtins import AssertionError
 from datetime import datetime
+from multiprocessing import Process
 from pathlib import Path
 from types import SimpleNamespace
 from typing import Optional
@@ -19,21 +22,24 @@ from datadiff.tools import assert_equal
 from pygments.formatters.terminal256 import Terminal256Formatter
 from pygments.lexers.data import YamlLexer
 from pygments.lexers.python import PythonLexer
+from streamlit import bootstrap
 from tabulate import tabulate
 from termcolor import colored
 
+from api.main import run_api
 from typhoon import connections
 from typhoon.cli_helpers.cli_completion import get_remote_names, get_dag_names, get_conn_envs, get_conn_ids, \
     get_var_types, get_deploy_targets, PROJECT_TEMPLATES, get_task_names
+from typhoon.cli_helpers.json_schema import generate_json_schemas
 from typhoon.cli_helpers.status import dags_with_changes, dags_without_deploy, check_connections_yaml, \
     check_connections_dags, check_variables_dags
 from typhoon.connections import Connection
+from typhoon.contrib.hooks.hook_factory import get_hook
 from typhoon.core import DagContext
-from typhoon.core.components import Component
-from typhoon.core.dags import DAGDefinitionV2, ArgEvaluationError, load_module_from_path
+from typhoon.core.dags import ArgEvaluationError, load_module_from_path
 from typhoon.core.glue import get_dag_errors, load_dag_definition
 from typhoon.core.settings import Settings
-from typhoon.deployment.packaging import build_all_dags
+from typhoon.deployment.packaging import build_all_dags, local_typhoon_path
 from typhoon.handler import run_dag
 from typhoon.introspection.introspect_extensions import get_typhoon_extensions, get_typhoon_extensions_info, \
     get_hooks_info
@@ -70,7 +76,7 @@ def set_settings_from_remote(remote: str):
 @click.group()
 def cli():
     """Typhoon CLI"""
-    pass
+    logging.getLogger('sqlitedict').setLevel(logging.CRITICAL)
 
 
 @cli.command()
@@ -103,8 +109,12 @@ def init(project_name: str, deploy_target: str, template: str):
             component_schema_path = (dest / 'component_schema.json')
 
         cfg_path.write_text(EXAMPLE_CONFIG.format(project_name=project_name, deploy_target=deploy_target))
-        dag_schema_path.write_text(DAGDefinitionV2.schema_json(indent=2))
-        component_schema_path.write_text(Component.schema_json(indent=2))
+        Settings.typhoon_home = dest
+        dag_schema, component_schema = generate_json_schemas()
+        dag_json_schema = json.dumps(dag_schema, indent=2)
+        dag_schema_path.write_text(dag_json_schema)
+        component_json_schema = json.dumps(component_schema, indent=2)
+        component_schema_path.write_text(component_json_schema)
 
         print(f'Project created in {dest}')
         print('If you want auto completion run the following:')
@@ -290,7 +300,11 @@ def build_dags(remote: Optional[str], dag_name: Optional[str], all_: bool):
         if Settings.deploy_target == 'typhoon':
             build_all_dags(remote=remote)
         else:
-            from typhoon.deployment.targets.airflow.airflow_build import build_all_dags_airflow
+            try:
+                from typhoon.deployment.targets.airflow.airflow_build import build_all_dags_airflow
+            except ModuleNotFoundError:
+                print('ERROR: Airflow is not installed. Try "pip install apache-airflow~=1.10"')
+                sys.exit(-1)
             build_all_dags_airflow(remote=remote)
     else:
         dag_errors = get_dag_errors().get(dag_name)
@@ -353,13 +367,7 @@ def cli_run_dag(remote: Optional[str], dag_name: str, execution_date: Optional[d
             print(tabulate(table_body, header, 'plain'), file=sys.stderr)
             sys.exit(-1)
 
-        dag_files = build_all_dags(remote=None, matching=dag_name)
-        assert len(dag_files) == 1, f'Expected to find exactly one dag file matching {dag_name}'
-        dag_file = dag_files[0]
-        # Sets the env variable for metadata store to the sqlite in CWD if not set, because the CWD will be different at
-        # runtime
-        # Settings.deploy_target = 'typhoon'
-        # Settings.metadata_db_url = Settings.metadata_db_url
+        build_all_dags(remote=None, matching=dag_name)
         run_local_dag(dag_name, execution_date)
     else:
         # TODO: Run lambda function
@@ -657,6 +665,16 @@ def remove_variable(remote: Optional[str], var_id: str):
     print(f'Variable {var_id} deleted')
 
 
+@cli.command(name='generate-json-schemas')
+def cli_generate_json_schemas():
+    """Generate JSON schemas using function data"""
+    dag_schema, component_schema = generate_json_schemas()
+    dag_json_schema = json.dumps(dag_schema, indent=2)
+    (Settings.typhoon_home/'dag_schema.json').write_text(dag_json_schema)
+    component_json_schema = json.dumps(component_schema, indent=2)
+    (Settings.typhoon_home/'component_schema.json').write_text(component_json_schema)
+
+
 @cli.group(name='extension')
 def cli_extension():
     """Manage Typhoon extensions"""
@@ -760,31 +778,30 @@ def run_in_subprocess(command: str, cwd: str):
 
 @cli.command()
 def webserver():
-    frontend = subprocess.Popen(
-        ["npm", "run", "serve"],
-        cwd=str(Path(__file__).parent.parent/'webserver/typhoon_webserver/frontend'))
+    api_process = Process(target=run_api)
+    api_process.start()
+    ui_script = Path(local_typhoon_path()).parent/'component_ui/component_builder.py'
     try:
-        sys.path.append(str(Path(__file__).parent.parent / 'webserver/typhoon_webserver/backend/'))
-        from core import app
-        app.run()
+        bootstrap.run(str(ui_script), f'run.py {ui_script}', [], {})
     finally:
-        frontend.kill()
+        api_process.terminate()
+        api_process.join()
 
 
 def transformations_locals():
-        custom_transformation_modules = {}
-        transformations_path = str(Settings.transformations_directory)
-        for filename in os.listdir(transformations_path):
-            if filename == '__init__.py' or not filename.endswith('.py'):
-                continue
-            module_name = filename[:-3]
-            module = load_module_from_path(
-                module_path=os.path.join(transformations_path, filename),
-                module_name=module_name,
-            )
-            custom_transformation_modules[module_name] = module
-        custom_transformations = SimpleNamespace(**custom_transformation_modules)
-        return custom_transformations
+    custom_transformation_modules = {}
+    transformations_path = str(Settings.transformations_directory)
+    for filename in os.listdir(transformations_path):
+        if filename == '__init__.py' or not filename.endswith('.py'):
+            continue
+        module_name = filename[:-3]
+        module = load_module_from_path(
+            module_path=os.path.join(transformations_path, filename),
+            module_name=module_name,
+        )
+        custom_transformation_modules[module_name] = module
+    custom_transformations = SimpleNamespace(**custom_transformation_modules)
+    return custom_transformations
 
 
 @cli.command()
@@ -818,6 +835,7 @@ Example usage:
         'dag_context': DagContext.from_cron_and_event_time('@daily', datetime.now(), granularity='day'),
         'transformations': transformations_locals(),
         'SimpleNamespace': SimpleNamespace,
+        'get_hook': get_hook,
     }
     start_ipython(argv=[], user_ns=user_ns, config=c)
 
